#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


"""Plugin for get film rating from CSFD (http://csfd.cz)
"""

from rating.rateall import register_plugin
import urllib2
import re


SEARCH_URL = \
'http://www.csfd.cz/hledani-filmu-hercu-reziseru-ve-filmove-databazi/?search='
MOVIE_PAGE = 'http://www.csfd.cz/film/'

RATING_RE = re.compile(r"<td\ style='[^>]+>\s*([0-9]+)%\s*</td>", re.I | re.S)



def find_movie_rating(page):
    """Searchs movie rating on downloaded film page."""
    try:
        return RATING_RE.search(page).group(1)
    except (AttributeError, IndexError):
        return None


def find_movie_page(search_res, mov_name, mov_year=None):
    """Searchs hyperlink to movie page from search results."""
    if not mov_name:
        return None
    mov_name = mov_name.replace(' ', '\ ')

    movpage_y_re = re.compile(r"""
    <a\ href=\"/film/([0-9]+-[^/]+)/\"[^>]+>{0}</a>\  \({1}\)<br>
    """.format(mov_name, mov_year), re.IGNORECASE | re.VERBOSE | re.DOTALL)
    movpage_re = re.compile(r"""
    <a\ href=\"/film/([0-9]+[^/]+)/\"[^>]+>{0}</a>
    """.format(mov_name), re.IGNORECASE | re.VERBOSE | re.DOTALL)

    if mov_year:
        match = movpage_y_re.search(search_res)
        if match:
            return match.group(1)
    
    match = movpage_re.search(search_res)
    if match:
        return match.group(1)
    return None


def rate_csfd(movies):
    """Gets movie rating from CSFD"""
    for movie in movies:
        if not 'MovieName' in movie or 'CSFDRating' in movie:
            continue

        # Download search results for given movie name.
        try:
            request = SEARCH_URL + urllib2.quote(movie['MovieName'])
        except (KeyError):
            continue    
        
        try:
            search_res = urllib2.urlopen(urllib2.Request(request)).read()
        except (urllib2.URLError, AttributeError):
            continue
        
        # Parse search result
        year = (('MovieYear' in movie) and movie['MovieYear']) or None
        csfdid = find_movie_page(search_res, movie['MovieName'], year)       
        if not csfdid:
            continue

        # Get movie page
        try:
            page = urllib2.urlopen(urllib2.Request(MOVIE_PAGE+csfdid)).read()
        except (urllib2.URLError, AttributeError):
            continue

        # Get movie rating
        rating = find_movie_rating(page)
        if rating:
            movie.update({'CSFDRating':rating})


register_plugin(rate_csfd)
